local _,L = ...
local rematch = Rematch
local pref

rematch:InitModule(function()
	pref = rematch.Dialog.Preferences
	pref.editBoxes = { "MinHP", "MaxHP", "MinXP", "MaxXP" } -- list of controls to tab through/update
	pref.MinHP.Label:SetText(L["Minimum Health"])
	pref.MinHP.tooltipTitle = L["Minimum Health"]
	pref.MinHP.tooltipBody = L["This is the minimum health preferred for a leveling pet.\n\nThe queue will prefer leveling pets with at least this much health (adjusted by expected damage taken if any chosen)."]
	pref.MinXP.Label:SetText(L["Minimum Level"])
	pref.MinXP.tooltipTitle = L["Minimum Level"]
	pref.MinXP.tooltipBody = L["This is the minimum level preferred for a leveling pet.\n\nLevels can be partial amounts. Level 4.33 is level 4 with 33% xp towards level 5."]
	pref.MaxHP.Label:SetText(L["Maximum Health"])
	pref.MaxHP.tooltipTitle = L["Maximum Health"]
	pref.MaxHP.tooltipBody = L["This is the maximum health preferred for a leveling pet."]
	pref.MaxXP.Label:SetText(L["Maximum Level"])
	pref.MaxXP.tooltipTitle = L["Maximum Level"]
	pref.MaxXP.tooltipBody = L["This is the maximum level preferred for a leveling pet.\n\nLevels can be partial amounts. Level 23.45 is level 23 with 45% xp towards level 24."]
	pref.AllowMM.tooltipTitle = format("%s %s & %s",L["Or any"],rematch:PetTypeAsText(6,16),rematch:PetTypeAsText(10,16))
	pref.AllowMM.tooltipBody = L["Allow low-health Magic and Mechanical pets to ignore the Minimum Health, since their racials allow them to often survive a hit that would ordinarily kill them."]
	pref.AllowMM.text:SetFontObject(GameFontNormal)
	pref.AllowMM.text:SetText(format("%s %s & %s",L["Or any"],rematch:PetTypeAsText(6,22),rematch:PetTypeAsText(10,22)))
end)

-- displays a tooltip of the preferences to a team
function rematch:ShowPreferencesTooltip(anchorTo,key)
	local team = RematchSaved[key]
	if not team then return end
	local info = rematch.info
	wipe(info)
	tinsert(info,format("%s %s","\124TInterface\\AddOns\\Rematch\\textures\\preference:16:16:0:0:64:64:18:45:18:45\124t",L["Leveling Preferences"]))
	if team.minHP then -- minimum hp defined
		tinsert(info,format("%s: %s%s",L["Minimum Health"],rematch.hexWhite,team.minHP))
		if team.expectedDD then -- expected damage taken
			tinsert(info,format(L["  For %s pets: %s%d"],rematch:PetTypeAsText(rematch.hintsOffense[team.expectedDD][1]),rematch.hexWhite,team.minHP*1.5))
			tinsert(info,format(L["  For %s pets: %s%d"],rematch:PetTypeAsText(rematch.hintsOffense[team.expectedDD][2]),rematch.hexWhite,team.minHP*2/3))
		end
		if team.allowMM then -- allow magic & mechanical
			tinsert(info,format("  %s %s & %s",L["Or any"],rematch:PetTypeAsText(6),rematch:PetTypeAsText(10)))
		end
	end
	if team.maxHP then -- maximum health
		tinsert(info,format("%s: %s%s",L["Maximum Health"],rematch.hexWhite,team.maxHP))
	end
	if team.minXP then -- minimum level
		tinsert(info,format("%s: %s%s",L["Minimum Level"],rematch.hexWhite,team.minXP))
	end
	if team.maxXP then -- maximum level
		tinsert(info,format("%s: %s%s",L["Maximum Level"],rematch.hexWhite,team.maxXP))
	end
	rematch:ShowTableTooltip(anchorTo,info)
end

function rematch:ShowPreferencesDialog(key)
	if not RematchSaved[key] then return end
	if rematch:IsDialogOpen("Preferences") and select(2,rematch:GetSideline())==key then
		rematch:HideDialog()
		return -- if dialog already up for this team, hide it and leave (toggle)
	end
	rematch:SetSideline(key,RematchSaved[key])
	local dialog = rematch:ShowDialog("Preferences",300,336,rematch:GetTeamTitle(key),nil,SAVE,rematch.PreferencesSave,CANCEL)
	pref:SetPoint("TOP",0,-36)
	pref:Show()
	if not pref.expectedDD then
		-- create expectedDD buttons the first time preference dialog is shown
		pref.expectedDD = {}
		for i=1,10 do
			local button = CreateFrame("CheckButton",nil,pref,"RematchSlotTemplate")
			button:SetID(i)
			button:SetSize(24,24)
			button.Icon:SetTexture("Interface\\Icons\\Icon_PetFamily_"..PET_TYPE_SUFFIX[i])
			button:SetPoint("CENTER",pref.ExpectedDDLabel,"BOTTOM",(i-1)*25-112,-20)
			button:SetCheckedTexture("Interface\\Buttons\\CheckButtonHilight","ADD")
			button:SetScript("OnClick",rematch.PreferencesExpectedDDOnClick)
			button:SetScript("OnEnter",rematch.PreferencesExpectedDDOnEnter)
			button:SetScript("OnLeave",rematch.HideTooltip)
			tinsert(pref.expectedDD,button)
		end
	end
	rematch:UpdatePreferencesDialog()
end

function rematch:UpdatePreferencesDialog()
	local team,key = rematch:GetSideline()

	for _,editBox in pairs(pref.editBoxes) do
		pref[editBox]:SetText(team[pref[editBox].var] or "")
		pref[editBox].Clear:SetShown(team[pref[editBox].var] and true)
	end
	pref.AllowMM:SetChecked(team.allowMM)
	local dd = team.expectedDD
	for i=1,10 do
		pref.expectedDD[i]:SetChecked(dd==i)
		pref.expectedDD[i].Icon:SetDesaturated(dd and dd~=i)
	end
end

function rematch:PreferencesOnTabPressed()
	local index = self:GetID()%#pref.editBoxes + 1
	pref[pref.editBoxes[index]]:SetFocus(true)
end

-- when text is typed into a preferences editbox to make sure it's a number)
-- this is used because setting numeric="true" won't allow decimals (ie 23.5)
function rematch:PreferencesOnChar()
	rematch:HideTooltip()
	if not tonumber(self:GetText()) then
		local team = rematch:GetSideline()
		if team then
			self:SetText(team[self.var] or "")
		end
	end
end

-- when text changes in an editbox, show/hide clear button and update team var
-- the OnChar will make sure the editbox has a valid value (or none at all)
function rematch:PreferencesOnTextChanged()
	local value = tonumber(self:GetText())
	self.Clear:SetShown(value and true or self:HasFocus())
	local team = rematch:GetSideline()
	if team then
		team[self.var] = value
	end
end

function rematch:PreferencesExpectedDDOnClick()
	local index = self:GetID()
	local team = rematch:GetSideline()
	if index==team.expectedDD then
		team.expectedDD = nil
	else
		team.expectedDD = index
	end
	rematch:UpdatePreferencesDialog()
end

-- tooltip of expected type buttons calculates damage expected
function rematch:PreferencesExpectedDDOnEnter()
	local petType = self:GetID()
	local team = rematch:GetSideline()
	local minHP = team.minHP
	if not minHP then -- no min health defined, show a tooltip to describe what the buttons are for
		rematch.ShowTooltip(self,L["Expected damage taken"],L["The minimum health of pets can be adjusted by the type of damage they are expected to receive."])
	else
		local info = rematch.info
		wipe(info)
		tinsert(info,format("%s: %s",L["Damage expected"],rematch:PetTypeAsText(petType)))
		tinsert(info,format("%s: \124cffffd200%s",L["Minimum Health"],minHP))
		tinsert(info,format(L["  For %s pets: \124cffffd200%d"],rematch:PetTypeAsText(rematch.hintsOffense[petType][1]),minHP*1.5))
		tinsert(info,format(L["  For %s pets: \124cffffd200%d"],rematch:PetTypeAsText(rematch.hintsOffense[petType][2]),minHP*2/3))
		rematch:ShowTableTooltip(self,info)
	end
end

-- save button clicked in preferences dialog
function rematch:PreferencesSave()
	rematch:HideDialog()
	rematch:PushPreferences()
	rematch:UpdateUI()
end

-- returns true if one of the team's preferences has a value
function rematch:HasPreferences(team)
	if team and (team.minHP or team.maxHP or team.minXP or team.maxXP) then
		return true
	end
end