local _,L = ...
local rematch = Rematch
local journal = RematchJournal
local settings

-- these are layouts for each numbered tab, with "left"/"right" in the index of a rematch.panel to show
-- from Main.lua panels are ordered: PetPanel, LoadoutPanel, TeamPanel, QueuePanel, OptionPanel
journal.layouts = {{"left","mid","right"},{"left","mid",nil,"right"},{"left","mid",nil,nil,"right"}}

rematch:InitModule(function()
	rematch.Journal = journal
	settings = RematchSettings
	settings.JournalPanel = settings.JournalPanel or 1
	rematch:SetupPanelTabs(journal.PanelTabs,settings.JournalPanel,L["Teams"],L["Queue"],L["Options"])
	for i=1,3 do
		journal.PanelTabs.Tabs[i]:SetScript("OnClick",journal.PanelTabOnClick)
	end
	-- if Collections already loaded, run OnEvent as if it's loading now
	if IsAddOnLoaded("Blizzard_Collections") then
		journal:OnEvent("ADDON_LOADED","Blizzard_Collections")
	else -- otherwise watch for journal to load before setting this stuff up
		journal:SetScript("OnEvent",journal.OnEvent)
		journal:RegisterEvent("ADDON_LOADED")
	end
	FloatingBattlePetTooltip.JournalClick:HookScript("OnClick",journal.ViewPetInJournal)
end)

-- prevent pet journal help plates from appearing on their own for first-time use

function journal:OnEvent(event,addon)
	if addon=="Blizzard_Collections" then
		if not settings.UseDefaultJournal and not GetCVarBitfield("closedInfoFrames",LE_FRAME_TUTORIAL_PET_JOURNAL) then
			SetCVarBitfield("closedInfoFrames",LE_FRAME_TUTORIAL_PET_JOURNAL,true)
		end
		self:UnregisterEvent("ADDON_LOADED")
		PetJournal:HookScript("OnShow",journal.ConfigureJournal)
		PetJournal:HookScript("OnHide",journal.DefaultJournalOnHide)
		journal.CloseButton:SetScript("OnClick",function() HideUIPanel(CollectionsJournal) end)
		SetPortraitToTexture(journal.portrait, "Interface\\Icons\\PetJournalPortrait")
		if IsAddOnLoaded("PetBattleTeams") then -- PBT does screwy stuff with framelevels; waiting a bit to reassert
			C_Timer.After(0.1,function() if journal:IsVisible() then journal:ConfigureJournal() end end)
		end
		local button = CreateFrame("CheckButton","UseRematchButton",PetJournal,"UICheckButtonTemplate,RematchTooltipScripts")
		button:SetSize(24,24)
		button:SetHitRectInsets(-2,-50,-2,-2)
		button.text:SetText(L["Rematch"])
		button.text:SetFontObject("GameFontNormal")
		button:SetPoint("LEFT",PetJournalSummonButton,"RIGHT",4,-1)
		button:SetScript("OnClick",function() settings.UseDefaultJournal=nil journal:ConfigureJournal() end)
		button:SetScript("OnEnter",function(self) if not rematch:UIJustChanged() then rematch.ShowTooltip(self) end end)
		button.tooltipTitle = L["Enable Rematch"]
		button.tooltipBody = L["Check this to use Rematch in the pet journal."]
	end
end

function journal:OnHide()
	rematch:HideWidgets()
	rematch:HideDialog()
	rematch.timeUIChanged = GetTime()
end

-- this returns the standalone frame when the journal hides
function journal:DefaultJournalOnHide()
	if journal.showStandaloneOnHide then
		rematch.Frame:Show()
		journal.showStandaloneOnHide = nil
	end
end

-- this is called in the PetJournal's OnShow and when journal tabs clicked; to set up the journal
-- hide is true when the journal needs hidden (typically due to it being on screen and in combat)
function journal:ConfigureJournal(hide)

	if rematch.Frame:IsVisible() then
		journal.showStandaloneOnHide = true
		rematch.Frame:Hide() -- hide standalone Frame (Frame and Journal can't coexist)
	end

	rematch:HideWidgets()
	rematch:HideDialog()
	rematch:HideNotes(settings.NotesNoESC)
	rematch.MiniPanel:Hide()

	UseRematchButton:Show()
	UseRematchButton:SetChecked(not settings.UseDefaultJournal)

	-- if UseDefaultJournal set, then hide our journal and leave
	if settings.UseDefaultJournal or hide or InCombatLockdown() then
		if not InCombatLockdown() then
			journal:SetParent(nil)
			journal:ClearAllPoints()
			journal:Hide()
			journal:SetShownExtras(true)
		else
			rematch:print("You are in combat. Try again when out of combat.")
			UseRematchButton:Hide()
		end
		return
	end

	rematch.timeUIChanged = GetTime()

	-- reparent our journal to the real one and set its level +1 higher than it to hide everything beneath
	journal:SetParent(PetJournal)
	journal:SetPoint("TOPLEFT")
	journal:SetFrameLevel(rematch:FindHighestFrameLevel(PetJournal)+1)

	journal:SetShownExtras(false) -- hide tutorial button and other stuff that stick out the edges

	rematch.BottomPanel:Reparent(journal)
	rematch.Toolbar:Reparent(rematch.TopPanel)
	rematch.TopPanel:Reparent(journal)
	rematch.TeamTabs:Reparent(journal)
	rematch.TeamTabs:SetShown(settings.JournalPanel==1 or settings.AlwaysTeamTabs)

	-- show/hide panels as defined in layout table
	local layout = journal.layouts[settings.JournalPanel]
	for i=1,5 do -- only checking first 5 panels
		local panel = _G[rematch.panels[i]]
		if layout[i] and panel then
			panel:SetParent(journal)
			panel:ClearAllPoints()
			panel:SetHeight(520)
			if panel.Resize then
				panel:Resize(280)
			end
			panel:SetPoint("BOTTOMLEFT",layout[i]=="left" and 4 or layout[i]=="mid" and 286 or 568,26)
			panel:Show()
		elseif panel then
			panel:Hide()
		end
	end

	rematch:UpdatePanelTabs(journal.PanelTabs)

	-- if PBT enabled, and team list is shown, minimize it
	if PetBattleTeams and not PetBattleTeams:GetModule("GUI"):GetIsMinimized() then
		PetBattleTeams:GetModule("GUI"):ToggleMinimize(true)
	end

	journal:Show()
	rematch:UpdateUI()
end

-- this hides/shows extras that stick out of the edges of the journal
function journal:SetShownExtras(show)
	PetJournalTutorialButton:SetShown(show)
	if PetBattleTeambuttonButtonIcon then
		PetBattleTeambuttonButtonIcon:GetParent():SetShown(show)
	end
end

-- one of the tabs (Teams, Queue, Options) clicked
function journal:PanelTabOnClick()
	settings.JournalPanel = self:GetID()
	rematch:SelectPanelTab(journal.PanelTabs,settings.JournalPanel)
	journal:ConfigureJournal()
end

-- from the "Click here to view in journal" of a linked pet
function journal:ViewPetInJournal()
	rematch.Roster:ClearAllFilters()
	local searchBox = rematch.PetPanel.Top.SearchBox
	searchBox:SetFocus(true)
	searchBox:SetText((C_PetJournal.GetPetInfoBySpeciesID(self:GetParent().speciesID)) or "")
	searchBox:ClearFocus()
end