local _,L = ...
local rematch = Rematch
local panel = RematchMiniPanel
local settings

rematch:InitModule(function()
	settings = RematchSettings
	rematch.MiniPanel = panel
	for i=1,3 do
		panel.Pets[i].HP:SetMinMaxValues(0,100)
		panel.Pets[i].XP:SetMinMaxValues(0,100)
		panel.Pets[i]:RegisterForClicks("AnyUp")
		panel.Pets[i]:RegisterForDrag("LeftButton")
		for j=1,3 do
			panel.Pets[i].Abilities[j].Arrow:SetTexCoord(1,0,0,0,1,1,0,1) -- rotate arrow
		end
	end
	panel:SetParent(RematchFrame)
	panel:SetPoint("TOPLEFT",4,-24-28)

	panel.Target.SaveStatus:SetText(L["This target has a saved team"])
	panel.Target.LoadButton:SetText(L["Load"])
	panel.Target.LoadButton.tooltipTitle = L["Load"]
	panel.Target.LoadButton.tooltipBody = L["Load the team saved for this target."]
	panel.timer = 0
	panel.elapsedLeaving = 0
end)

function panel:Update()
	local info,petID = rematch.info
	for i=1,3 do
		petID,info[1],info[2],info[3] = C_PetJournal.GetPetLoadOutInfo(i)
		local button = panel.Pets[i]
		rematch:FillPetSlot(button,petID)
		if petID then
			-- fill in abilities
			for j=1,3 do
				rematch.LoadoutPanel:FillAbilityButton(button.Abilities[j],petID,info[j])
			end
			-- xp bars
			local _,_,level,xp,maxXP,_,_,_,_,petType = C_PetJournal.GetPetInfoByPetID(petID)
			if level<25 then
				button.XP:Show()
				button.XP:SetValue(xp/maxXP*100)
				button.HP:SetPoint("TOP",button,"BOTTOM",0,-9)
			else
				button.XP:Hide()
				button.HP:SetPoint("TOP",button,"BOTTOM",0,-19)
			end
			-- hp bar
			local hp,maxHP = C_PetJournal.GetPetStats(petID)
			local hpPercent = hp/maxHP*100
			button.HP:SetValue(hpPercent)
			if level==25 then
				button.HP.Icon:Show()
				button.HP.Text:Show()
				button.HP.Text:SetText(format("%d%%",hpPercent))
			else
				button.HP.Icon:Hide()
				button.HP.Text:Hide()
			end
			button.queueControl = rematch:IsSlotQueueControlled(i)
			button.Leveling:SetShown(button.queueControl)
		end
	end
	panel:UpdateTarget()
end

function panel:UpdateTarget(unit,npcID)
	if not panel:IsVisible() or (not settings.Minimized and not settings.NarrowMode) then
		return
	end
	if not unit and UnitExists("target") then -- this update is not being called during a PLAYER_TARGET_CHANGED
		_,npcID = rematch:GetUnitNameandID("target")
	end
	local saved = RematchSaved
	local parent = panel:GetParent()
	if npcID and saved[npcID] and settings.loadedTeam~=npcID then
		if parent==rematch.Frame then
			parent:SetHeight(218+28)
		else
			parent:SetHeight(417-62)
		end
		panel.Target:Show()
		rematch.LoadoutPanel:UpdateTargetModelandPets(panel.Target,"target",npcID,true)
	else
		if parent==rematch.Frame then
			rematch.Frame:SetHeight(156+28)
		else
			parent:SetHeight(417)
		end
		panel.Target:Hide()
	end
end

-- click of one of the ability buttons beside each pet
function panel:AbilityOnClick()
	if rematch.ChatLinkAbility(self) then
		return -- was only linking ability, can leave
	end
	-- check if flyout already open for this ability; close it and leave if so
	if panel.Flyout:IsVisible() and panel.Flyout:GetParent()==self then
		rematch:HideFlyout()
		return
	end
	rematch:HideWidgets()
	local petSlot = self:GetParent():GetID()
	local abilitySlot = self:GetID()
	panel.Flyout:SetParent(self)
	panel.Flyout:SetFrameStrata("DIALOG")
	panel.Flyout:SetHeight(settings.ShowAbilityNumbers and 46 or 36)
	for i=1,2 do
		panel.Flyout.Numbers[i]:SetShown(settings.ShowAbilityNumbers)
	end
	panel.Flyout:SetPoint("TOPRIGHT",self,"TOPLEFT",0,5)
	panel.Flyout:Show()
	local info,petID = rematch.info
	wipe(info)
	petID,info[1],info[2],info[3] = C_PetJournal.GetPetLoadOutInfo(petSlot)
	if not petID then return end
	C_PetJournal.GetPetAbilityList((C_PetJournal.GetPetInfoByPetID(petID)),rematch.abilityList,rematch.levelList)
	panel.Flyout.petID = petID
	for i=1,2 do
		local listIndex = (i-1)*3+abilitySlot
		local abilityID = rematch.abilityList[listIndex]
		rematch.LoadoutPanel:FillAbilityButton(panel.Flyout.Abilities[i],petID,abilityID)
	end
end

function panel:FlyoutAbilityOnClick()
	if rematch.ChatLinkAbility(self) then
		return -- only linking ability to chat, leave
	end
	local petSlot = self:GetParent():GetParent():GetParent():GetID()
	local abilitySlot = self:GetParent():GetParent():GetID()
	if self.Cover:IsVisible() then
		return
	else
		rematch.timeUIChanged = GetTime()
		self:GetParent():Hide()
		C_PetJournal.SetAbility(petSlot,abilitySlot,self.abilityID)
	end
end

function panel:Reparent(parent)
	panel:SetParent(parent)
	panel:ClearAllPoints()
	if parent==rematch.Frame then -- when anchoring to Frame then it's minimized view
		panel:SetPoint("TOPLEFT",4,-24-28)
		panel.TopTileStreaks:Show()
		panel:Resize(260)
	else -- otherwise it's for one of the panels
		if settings.NarrowMode then
			panel:SetPoint("TOPLEFT",rematch.Frame,"TOPLEFT",4,-24-28)
		else
			panel:SetPoint("BOTTOMLEFT",parent,"TOPLEFT",0,2)
			panel.Target:Hide()
		end
		panel.TopTileStreaks:Hide()
		if settings.NarrowMode and parent:GetParent()==rematch.Frame then
			panel:Resize(337) -- narrow mode above pets, teams or queue
		else
			panel:Resize(280) -- standalone normal mode above queue
		end
	end
	panel:Show()
end

-- possible widths: 260 (minimized), 280 (above queue panel in normal view), 337 (above all panels in narrowmode)
function panel:Resize(width)
	panel:SetWidth(width)
	local xwidth, xoff = 0,0
	if width==260 then -- minimized
		xwidth, xoff = 84, 8
	elseif width==280 then -- above queue in normal view
		xwidth, xoff = 88, 14
	else -- above pets, teams and queue in narrowmode
		xwidth, xoff = 102, 28
	end
	for i=1,3 do
		panel.Pets[i]:SetPoint("TOPLEFT",(i-1)*xwidth+xoff,-12)
		panel.Glow.Overlays[i]:SetPoint("TOPLEFT",panel.Pets[i],-3,3)
	end
end
